pub enum CacheError<E, S> {
Error(E),
IncompatibleState {
want: S,
},
}Expand description
Wrapper around other errors to indicate that typechecking or applying program transformations failed because the source has not been parsed yet.
# Type parameters
E: the underlying, wrapped error typeS: the entry state, whether TermEntryState or AstEntryState in practice.
Variants
Error(E)
IncompatibleState
The state of the entry in the cache is incompatible with the requested operation.
Fields
want: SImplementations
Sourceimpl<E, S> CacheError<E, S>
impl<E, S> CacheError<E, S>
pub fn unwrap_error(self, msg: &str) -> E
pub fn map_err<O>(self, f: impl FnOnce(E) -> O) -> CacheError<O, S>
Sourcepub fn force_cast<T>(self) -> CacheError<E, T>
pub fn force_cast<T>(self) -> CacheError<E, T>
Assuming that self is of the form CacheError::Error(e), cast the error type to another
arbitrary state type T.
Panic
This method panics if self is CacheError::IncompatibleState.