pub enum FileCloseError {
FileIdNotFound,
FileNotOpen,
}Expand description
The errors that can occur while closing an in memory file.
Variants
FileIdNotFound
The file was not closed because no mapping of the source path to a FileId could be found.
FileNotOpen
A file with the given path was found, but it was not open in memory.