pub enum SourcePath {
Path(PathBuf, InputFormat),
Snippet(PathBuf),
Std(StdlibModule),
Query,
ReplInput(usize),
ReplTypecheck,
ReplQuery,
CliFieldAssignment,
Override(FieldPath),
Generated(String),
}Expand description
Input data usually comes from files on the file system, but there are also lots of cases where we want to synthesize other kinds of inputs.
Note that a SourcePath does not uniquely identify a cached input:
- Some functions (like SourceCache::add_file) add a new cached input unconditionally.
SourceCache::get_or_add_filewill add a new cached input at the sameSourcePathif the file on disk was updated.
The equality checking of SourcePath only affects SourceCache::replace_string, which
overwrites any previous cached input with the same SourcePath.
Variants
Path(PathBuf, InputFormat)
A file at the given path.
Note that this does not need to be a real file on the filesystem: it could still be loaded
from memory by, e.g, SourceCache::add_string.
This is the only SourcePath variant that can be resolved as the target of an import
statement.
Snippet(PathBuf)
A subrange of a file at the given path.
This is used by NLS to analyze small parts of files that don’t fully parse. The original file path is preserved, because it’s needed for resolving imports.
Std(StdlibModule)
Query
ReplInput(usize)
ReplTypecheck
ReplQuery
CliFieldAssignment
Override(FieldPath)
Generated(String)