pub enum TermEntryState {
Populated,
CustomTransforming,
CustomTransformed {
transform_id: usize,
},
ImportsResolving,
ImportsResolved,
Transforming,
Transformed,
Closurized,
}Expand description
The state of an entry of the term cache.
Imports
Usually, when applying a procedure to a term entry (e.g. program transformations), we process
all of its transitive imports as well. We start by processing the entry, updating the state to
XXXing (ex: Typechecking) upon success. Only when all the imports have been successfully
processed, the state is updated to XXXed (ex: Typechecked).
Variants
Populated
The initial state. The term is in the cache but hasn’t been processed further yet.
CustomTransforming
A custom transformation of the entry (through Program::custom_transform) is underway.
CustomTransformed
This entry has completed custom transformations of this ID and lower.
ImportsResolving
The imports of the entry have been resolved, and the imports of its (transitive) imports are being resolved.
ImportsResolved
The imports of the entry and its transitive dependencies has been resolved.
Transforming
The entry have been transformed, and its (transitive) imports are being transformed.
Transformed
The entry and its transitive imports have been transformed.
Closurized
The entry has been closurized.