pub struct SimpleResolver { /* private fields */ }Expand description
Resolve imports from a mockup file database. Used to test imports without accessing the
file system. File name are stored as strings, and silently converted from/to OsString
when needed: don’t use this resolver with source code that import non UTF-8 paths.
Implementations
Sourceimpl SimpleResolver
impl SimpleResolver
pub fn new() -> SimpleResolver
Sourcepub fn add_source(&mut self, name: String, source: String)
pub fn add_source(&mut self, name: String, source: String)
Add a mockup file to available imports.