pub struct AstCache { /* private fields */ }Expand description
The self-referencing struct.
Implementations
Sourceimpl AstCache
impl AstCache
Sourcepub fn contains(&self, file_id: FileId) -> bool
pub fn contains(&self, file_id: FileId) -> bool
Returns true if the AST cache contains an entry for the given file id.
Sourcepub fn get_alloc(&self) -> &AstAlloc
pub fn get_alloc(&self) -> &AstAlloc
Returns the underlying allocator, which might be required to call various helpers.
Sourcepub fn get_entry(&self, file_id: FileId) -> Option<&AstEntry<'_>>
pub fn get_entry(&self, file_id: FileId) -> Option<&AstEntry<'_>>
Returns a reference to a cached AST entry.
Sourcepub fn get_wildcards(&self, file_id: FileId) -> Option<&Wildcards<'_>>
pub fn get_wildcards(&self, file_id: FileId) -> Option<&Wildcards<'_>>
Returns the wildcards associated to an entry.
Sourcepub fn entry_state(&self, file_id: FileId) -> Option<AstEntryState>
pub fn entry_state(&self, file_id: FileId) -> Option<AstEntryState>
Retrieves the state of an entry. Returns None if the entry is not in the AST cache.
Sourcepub fn update_state(
&mut self,
file_id: FileId,
new: AstEntryState,
) -> Result<AstEntryState, TermNotFound>
pub fn update_state( &mut self, file_id: FileId, new: AstEntryState, ) -> Result<AstEntryState, TermNotFound>
Updates the state of an entry and returns the previous state, or an error if the entry isn’t in the cache.
Sourcepub fn parse_nickel<'ast>(
&'ast mut self,
file_id: FileId,
source: &str,
) -> Result<&'ast Ast<'ast>, ParseErrors>
pub fn parse_nickel<'ast>( &'ast mut self, file_id: FileId, source: &str, ) -> Result<&'ast Ast<'ast>, ParseErrors>
Parses a Nickel expression and stores the corresponding AST in the cache.
Sourcepub fn parse_nickel_repl<'ast>(
&'ast mut self,
file_id: FileId,
source: &str,
) -> Result<ExtendedTerm<Ast<'ast>>, ParseErrors>
pub fn parse_nickel_repl<'ast>( &'ast mut self, file_id: FileId, source: &str, ) -> Result<ExtendedTerm<Ast<'ast>>, ParseErrors>
Same as Self::parse_nickel but accepts the extended syntax allowed in the REPL.
Caution: this method doesn’t cache the potential id of a top-level let binding, although it does save the bound expression, which is required later for typechecking, program transformation, etc.
pub fn remove(&mut self, file_id: FileId) -> Option<AstEntry<'_>>
Sourcepub fn typecheck(
&mut self,
slice: CacheHubView<'_>,
file_id: FileId,
initial_mode: TypecheckMode,
) -> Result<CacheOp<()>, AstCacheError<TypecheckError>>
pub fn typecheck( &mut self, slice: CacheHubView<'_>, file_id: FileId, initial_mode: TypecheckMode, ) -> Result<CacheOp<()>, AstCacheError<TypecheckError>>
Typechecks an entry of the cache and updates its state accordingly, or does nothing if the entry has already been typechecked. Requires that the corresponding source has been parsed. Note that this method currently fail on a non-Nickel file, that can’t have been parsed to an AST.
If the source contains imports, recursively typecheck on the imports too.
RFC007
During the transition period between the old VM and the new bytecode VM, this method performs typechecking on the new representation crate::ast::Ast.
Sourcepub fn typecheck_stdlib(
&mut self,
slice: CacheHubView<'_>,
) -> Result<CacheOp<()>, AstCacheError<TypecheckError>>
pub fn typecheck_stdlib( &mut self, slice: CacheHubView<'_>, ) -> Result<CacheOp<()>, AstCacheError<TypecheckError>>
Typechecks the stdlib. This has to be public because it’s used in benches. It probably does not have to be used for something else.
Sourcepub fn type_of(
&mut self,
slice: CacheHubView<'_>,
file_id: FileId,
) -> Result<CacheOp<Type<'_>>, AstCacheError<TypecheckError>>
pub fn type_of( &mut self, slice: CacheHubView<'_>, file_id: FileId, ) -> Result<CacheOp<Type<'_>>, AstCacheError<TypecheckError>>
Typechecks a file (if it wasn’t already) and returns the inferred type, with type wildcards properly substituted.
Sourcepub fn add_type_binding(
&mut self,
slice: CacheHubView<'_>,
id: LocIdent,
file_id: FileId,
) -> Result<(), AstCacheError<Infallible>>
pub fn add_type_binding( &mut self, slice: CacheHubView<'_>, id: LocIdent, file_id: FileId, ) -> Result<(), AstCacheError<Infallible>>
Adds a binding to the type environment. The bound term is identified by its file id
file_id.
Sourcepub fn add_type_bindings(
&mut self,
pos_table: &PosTable,
slice: CacheHubView<'_>,
term: &NickelValue,
) -> Result<(), NotARecord>
pub fn add_type_bindings( &mut self, pos_table: &PosTable, slice: CacheHubView<'_>, term: &NickelValue, ) -> Result<(), NotARecord>
Add the bindings of a record to the type environment. Ignore fields whose name are defined through interpolation.