pub struct AstResolver<'ast, 'cache> { /* private fields */ }Expand description
As RFC007 is being rolled out, the typechecker now needs to operate on the new AST. We need a structure that implements AstImportResolver.
For borrowing reasons, this can’t be all of CacheHub or all of ast_cache::AstCache, as we
need to split the different things that are borrowed mutably or immutably. AstResolver is a
structure that borrows some parts of the cache during its lifetime and will retrieve alredy
imported ASTs, or register the newly imported ones in a separate hashmap that can be added back
to the original cache once import resolution is done.
Implementations
Sourceimpl<'ast, 'cache> AstResolver<'ast, 'cache>
impl<'ast, 'cache> AstResolver<'ast, 'cache>