pub struct FileCloseResult {
pub closed_id: FileId,
pub replacement_id: Result<FileId, Error>,
}Expand description
Contains information about the closed in-memory file and its replacement from the filesystem in the case that an in-memory file was closed successfully.
Fields
closed_id: FileIdThe FileId of the in-memory file that was closed.
replacement_id: Result<FileId, Error>The FileId of the file loaded from the filesystem with the same path as the closed file, or an error indicating why the file could not be opened. An error would be expected here in the case that the file was deleted, which would also send a close file notification to the LSP.