pub struct ImportData {
pub imports: HashMap<FileId, HashSet<ImportTarget>>,
pub rev_imports: HashMap<FileId, HashMap<FileId, TermPos>>,
}Expand description
Stores dependencies and reverse dependencies data between sources.
Fields
imports: HashMap<FileId, HashSet<ImportTarget>>A map containing for each FileId a list of files they import (directly).
rev_imports: HashMap<FileId, HashMap<FileId, TermPos>>A map containing for each FileId a list of files importing them (directly). Note that we don’t need to store the format here, as only Nickel files can import other files. We do however store the position of the first import expression (the same file can be imported many times from a given file), for error reporting purpose.
Implementations
Sourceimpl ImportData
impl ImportData
pub fn new() -> Self
Sourcepub fn imports(&self, file: FileId) -> impl Iterator<Item = FileId> + '_
pub fn imports(&self, file: FileId) -> impl Iterator<Item = FileId> + '_
Returns the set of files that this file imports.
Sourcepub fn rev_imports(&self, file: FileId) -> impl Iterator<Item = FileId> + '_
pub fn rev_imports(&self, file: FileId) -> impl Iterator<Item = FileId> + '_
Returns the set of files that import this file.
Sourcepub fn transitive_rev_imports(&self, file: FileId) -> HashSet<FileId>
pub fn transitive_rev_imports(&self, file: FileId) -> HashSet<FileId>
Returns the set of files that transitively depend on this file.
Sourcepub fn transitive_imports(&self, file: FileId) -> HashSet<FileId>
pub fn transitive_imports(&self, file: FileId) -> HashSet<FileId>
Returns the set of files that this file transitively depends on.