pub struct PointedExportErrorData {
pub path: NickelPointer,
pub error: ExportErrorKind,
}Expand description
The type of an error occurring during serialization, together with a path to the field that contains a non-serializable value.
Fields
path: NickelPointerThe path to the field that contains a non-serializable value. This might be empty if the error occurred before entering any record.
error: ExportErrorKindThe cause of the error.
Implementations
Sourceimpl PointedExportErrorData
impl PointedExportErrorData
Sourcepub fn with_elem(self, elem: NickelPointerElem) -> PointedExportErrorData
pub fn with_elem(self, elem: NickelPointerElem) -> PointedExportErrorData
Pushes a NickelPointerElem to the end of the path of this export error.
Sourcepub fn with_pos_table(self, pos_table: PosTable) -> ExportErrorData
pub fn with_pos_table(self, pos_table: PosTable) -> ExportErrorData
Attaches a position table to this error, producing a ExportError.