pub enum ValueContentRefMut<'a> {
Show 14 variants
Null(&'a mut NickelValue),
Bool(&'a mut NickelValue),
Number(&'a mut NumberData),
Array(Container<&'a mut ArrayData>),
Record(Container<&'a mut RecordData>),
String(&'a mut StringData),
Thunk(&'a mut Thunk),
Term(&'a mut TermData),
Label(&'a mut LabelData),
EnumVariant(&'a mut EnumVariantData),
ForeignId(&'a mut ForeignIdData),
SealingKey(&'a mut SealingKeyData),
CustomContract(&'a mut CustomContractData),
Type(&'a mut TypeData),
}Expand description
Mutable version of ValueContentRef.
Variants
Null(&'a mut NickelValue)
Given the encoding of inline values, it’s a bad idea to provide a bare Rust reference to the underlying data encoding the inline value. While it’s possible currently, we might use a more exotic layout in the future making it impossible to produce a valid Rust reference to an InlineValue.
A mutable reference is mostly useful for value blocks anyway. For inline values, we just return the original value back. It can be overridden directly with a new inline value.
Note that the empty array case and the empty record case are handled by the Container part in Self::Array and Self::Record.
Bool(&'a mut NickelValue)
Number(&'a mut NumberData)
Array(Container<&'a mut ArrayData>)
Record(Container<&'a mut RecordData>)
String(&'a mut StringData)
Thunk(&'a mut Thunk)
Term(&'a mut TermData)
Label(&'a mut LabelData)
EnumVariant(&'a mut EnumVariantData)
ForeignId(&'a mut ForeignIdData)
SealingKey(&'a mut SealingKeyData)
CustomContract(&'a mut CustomContractData)
Type(&'a mut TypeData)