pub struct FieldOverride {
pub path: FieldPath,
pub value: String,
pub priority: MergePriority,
}Expand description
Several CLI commands accept additional overrides specified directly on the command line. They are represented by this structure.
Fields
path: FieldPathThe field path identifying the (potentially nested) field to override.
value: StringThe overriding value.
priority: MergePriorityThe priority associated with this override.
Implementations
Sourceimpl FieldOverride
impl FieldOverride
Sourcepub fn parse(
cache: &mut CacheHub,
assignment: String,
priority: MergePriority,
) -> Result<Self, ParseError>
pub fn parse( cache: &mut CacheHub, assignment: String, priority: MergePriority, ) -> Result<Self, ParseError>
Parse an assignment path.to.field=value to a field override, with the priority given as a
separate argument.
Internally, the parser entirely parses the value part to a NickelValue (have it accept
anything after the equal sign is in fact harder than actually parsing it), but what we need
at this point is just a string. Thus, parse uses the span to extract back the value
part of the input string.
Theoretically, this means we parse two times the same string (the value part of an assignment). In practice, we expect this cost to be completely negligible.
Selectors
The value part accepts special selectors starting with a leading @ that aren’t part of
the core Nickel syntax. This list is subject to extensions.
foo.bar=@env:<var>will extract a string value from the environment variable<var>and put it infoo.bar.