pub struct FieldPath(pub Vec<LocIdent>);Expand description
A path of fields, that is a list, locating this field from the root of the configuration.
Tuple Fields
0: Vec<LocIdent>Implementations
Sourceimpl FieldPath
impl FieldPath
pub fn new() -> Self
Sourcepub fn parse(caches: &mut CacheHub, input: String) -> Result<Self, ParseError>
pub fn parse(caches: &mut CacheHub, input: String) -> Result<Self, ParseError>
Parse a string as a query path. A query path is a sequence of dot-separated identifiers. Identifiers can be enclosed by double quotes when they contain characters that aren’t allowed inside bare identifiers. The accepted grammar is the same as a sequence of record accesses in Nickel, although string interpolation is forbidden.
Post-conditions
If this function succeeds and returns Ok(field_path), then field_path.0 is non empty.
Indeed, there’s no such thing as a valid empty field path (at least from the parsing point
of view): if input is empty, or consists only of spaces, parse returns a parse error.
Sourcepub fn parse_opt(
cache: &mut CacheHub,
input: Option<String>,
) -> Result<Self, ParseError>
pub fn parse_opt( cache: &mut CacheHub, input: Option<String>, ) -> Result<Self, ParseError>
As Self::parse, but accepts an Option to accomodate for the absence of path. If the
input is None, Ok(FieldPath::default()) is returned (that is, an empty field path).