pub struct InputParser { /* private fields */ }Expand description
Validator enabling multiline input.
The behavior is the following:
- always end an input that starts with the command prefix
: - otherwise, try to parse the input. If an unexpected end of file error occurs, continue the input in a new line. Otherwise, accept and end the input.
Implementations
Sourceimpl InputParser
impl InputParser