pub struct TermCache { /* private fields */ }Expand description
The term cache stores the parsed values (the runtime representation) of sources.
Implementations
Sourceimpl TermCache
impl TermCache
pub fn new() -> Self
Sourcepub fn update_state(
&mut self,
file_id: FileId,
new: TermEntryState,
) -> Result<TermEntryState, TermNotFound>
pub fn update_state( &mut self, file_id: FileId, new: TermEntryState, ) -> Result<TermEntryState, TermNotFound>
Updates the state of an entry and returns the previous state, or an error if the entry isn’t in the cache.
Sourcepub fn entry_state(&self, file_id: FileId) -> Option<TermEntryState>
pub fn entry_state(&self, file_id: FileId) -> Option<TermEntryState>
Retrieves the state of an entry. Returns None if the entry is not in the term cache. This
might happen if the file hasn’t been parsed, or if the term cache hasn’t be filled from the
AST cache yet. The latter is supposed to happen right before program transformations.
Sourcepub fn closurize<C: EvalCache>(
&mut self,
cache: &mut C,
import_data: &ImportData,
file_id: FileId,
) -> Result<CacheOp<()>, TermCacheError<()>>
pub fn closurize<C: EvalCache>( &mut self, cache: &mut C, import_data: &ImportData, file_id: FileId, ) -> Result<CacheOp<()>, TermCacheError<()>>
Replaces a cache entry by a closurized version of itself. If it contains imports, closurize them recursively.
Closurization is not required before evaluation, but it has two benefits:
- the closurized term uses the evaluation cache, so if it is imported in multiple places then they will share a cache
- the eval cache’s built-in mechanism for preventing infinite recursion will also apply to recursive imports.
The main disadvantage of closurization is that it makes the resulting runtime representation less useful. You wouldn’t want to closurize before pretty-printing, for example. This isn’t as important these days, since we also have the AST representation at hand.
Sourcepub fn terms(&self) -> &HashMap<FileId, TermEntry>
pub fn terms(&self) -> &HashMap<FileId, TermEntry>
Returns an immutable reference to the whole term cache.
Sourcepub fn get_owned(&self, file_id: FileId) -> Option<NickelValue>
pub fn get_owned(&self, file_id: FileId) -> Option<NickelValue>
Retrieves a fresh clone of a cached term.
Sourcepub fn get(&self, file_id: FileId) -> Option<&NickelValue>
pub fn get(&self, file_id: FileId) -> Option<&NickelValue>
Retrieves a reference to a cached term.
Sourcepub fn get_entry(&self, file_id: FileId) -> Option<&TermEntry>
pub fn get_entry(&self, file_id: FileId) -> Option<&TermEntry>
Retrieves the whole entry for a given file id.