pub enum MultiStringToken<'input> {
Error,
Literal(String),
CandidateEnd(&'input str),
CandidateInterpolation(&'input str),
QuotesCandidateInterpolation(&'input str),
End,
Interpolation,
}Expand description
The tokens in multiline string mode.
Variants
Error
Literal(String)
CandidateEnd(&'input str)
A candidate end. A multiline string starting delimiter MultiStringStart can have a
variable number of % character, so the lexer matches candidate end delimiter, compare the
number of characters, and either emit the End token above, or turn the CandidateEnd to a
FalseEnd otherwise
CandidateInterpolation(&'input str)
Same as CandidateEnd, but for interpolation
QuotesCandidateInterpolation(&'input str)
Unfortunate consequence of Logos’
issue#200. The other rules should be
sufficient to match this as a double quote followed by a CandidateInterpolation, but if we
omit this token, the lexer can fail unexpectedly on valid inputs because of #200.
End
Token emitted by the modal lexer for the parser once it has decided that a CandidateEnd is
an actual end token.
Interpolation